#!/usr/bin/env python
import paramiko, wx, os
from contextlib import contextmanager
from stat import S_ISDIR, S_ISREG
from dialogs import CSDialog, IODialog, OKDialog
import wx.lib.scrolledpanel as scrolled

class SSH():
    def __init__(self, dplug):
        self.dplug = dplug
        self.gui = dplug.main_class
        
        self.filedir = self.dplug.homedir

        #Conf
        self.dplug.plugin_config.add_section("network")

    def get_bmp(self, file, ReturnAsImage = False, parent = ""):
        path = os.path.join(self.dplug.directory, "icons", file)
        bmp = wx.Image(path, wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        if ReturnAsImage:
            return bmp
        else:
            return wx.BitmapButton(parent, id = wx.ID_ANY, bitmap = bmp, size = (40, 40))
    
    def ssh_session(self):
        sshpage = wx.Panel(self.gui.notebook)
        siz = wx.BoxSizer(wx.VERTICAL)
        self.local = wx.ListCtrl(sshpage)
        buttons = wx.BoxSizer(wx.HORIZONTAL)
        self.upload = self.get_bmp("up.png", False, sshpage)
        self.open = self.get_bmp("open.png", False, sshpage)
        self.download = self.get_bmp("down.png", False, sshpage)
        buttons.AddSpacer((0, 0), 1, wx.EXPAND, 5)
        buttons.Add(self.upload)
        buttons.Add(self.open)
        buttons.Add(self.download)
        buttons.AddSpacer(( 0, 0), 1, wx.EXPAND, 5 )
        self.virtual = wx.ListCtrl(sshpage)
        siz.Add(self.virtual, -1, wx.EXPAND)
        siz.Add(buttons, 0, wx.EXPAND)
        siz.Add(self.local, -1, wx.EXPAND)
        sshpage.SetSizer(siz)
        
        try:
            self.local_update()
            self.virtual_update()
        except:
            sshpage.Destroy()
            OKDialog("SSH error", "SSH is not started yet!")
        else:
            self.gui.add_tab("SSH Session", sshpage)
            
            self.gui.Bind(wx.EVT_BUTTON, lambda event: self.get_upload_files(), self.upload)
            self.gui.Bind(wx.EVT_BUTTON, lambda event: self.get_open_files(), self.open)
            self.gui.Bind(wx.EVT_BUTTON, lambda event: self.get_download_files(), self.download)
            self.local.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.localdbclick)
            self.virtual.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.virtualdbclick)
            self.local.Bind(wx.EVT_ENTER_WINDOW, lambda event: self.focus("local"))
            self.virtual.Bind(wx.EVT_ENTER_WINDOW, lambda event: self.focus("virtual"))
            
            self.gui.Layout()
    
    def focus(self, panel = "virtual"):
        if panel == "virtual":
            self.open.Disable()
            self.upload.Disable()
            self.download.Enable()
        elif panel == "local":
            self.open.Enable()
            self.upload.Enable()
            self.download.Disable()
    
    def local_update(self):
        self.local.ClearAll()
        self.local.InsertStringItem(0, "[Back]")
        listdir = os.listdir(str(self.filedir))
        listdir.sort(key = str.lower)
        for i, v in enumerate(listdir):
            if(os.path.isdir(os.path.join(self.filedir,v))):
                self.local.InsertStringItem(i+1, "[F] " + v)
            else:
                self.local.InsertStringItem(i+1, v)
    
    def virtual_update(self):
        self.virtual.ClearAll()
        self.virtual.InsertStringItem(0, "[Back]")
        try:
            listdir = self.sftp.listdir()
        except IOError:
            OKDialog("SSH Session", "There occured an error;\nFrequent error: 'Permission Denied'")
        else:
            listdir.sort(key = str.lower)
            for i, v in enumerate(listdir):
                mode = self.sftp.stat(os.path.join(self.sftp.getcwd(), v)).st_mode
                if S_ISDIR(mode):
                    self.virtual.InsertStringItem(i+1, "[F] " + v)
                elif S_ISREG(mode):
                    self.virtual.InsertStringItem(i+1, v)

    def localdbclick(self, event):
        selection = event.GetLabel()
        ev = selection.strip().split(" ")[1::]
        ev = ' '.join(ev)
        if selection.startswith("[F] "):
            self.filedir = os.path.join(self.filedir, ev)
            self.local_update()
        elif "[Back]" in selection:
            if os.path.split(self.filedir)[1] == "":
                pass
            else:
                self.filedir = os.path.split(self.filedir)[0]
            self.local_update()
        else:
            #OKDialog("SSH Session", "You didn't define a double click behaviour yet, goto configuratio...")
            # upload or open file  ...
            #self.gui.ded.editfl(self.filedir + "/" + selection)
            pass

    def virtualdbclick(self, event):
        selection = event.GetLabel()
        ev = selection.strip().split(" ")[1::]
        ev = ' '.join(ev)
        if selection.startswith("[F] "):
            self.sftp.chdir(ev)
            self.virtual_update()
        elif "[Back]" in selection:
            self.sftp.chdir(os.path.split(self.sftp.getcwd())[0])
            self.virtual_update()
        else:
            self.download_file(ev)
    
    def ssh_switch(self):
        if self.dplug.plugins.network.ssh_status.IsChecked():
            if not self.dplug.plugin_config.get("network","ssh_host") or \
            not self.dplug.plugin_config.get("network","ssh_username") or \
            not self.dplug.plugin_config.get("network", "ssh_password"):  x = self.install()
            else: x = True
            if x: self.start_ssh()
        else:
            self.stop_ssh()

    def Command(self, cmd, next):
        if cmd == "ssh":
            next = next.strip().split(" ")
            if next[0] == "connect" or next[0] == "start":
                self.start_ssh()
            elif next[0] == "disconnect" or next[0] == "stop":
                self.stop_ssh()
            elif next[0] == "show":
                if next[1] == "download":
                    self.download()
                elif next[1] == "upload":
                    self.upload()
            elif next[0] == "hide":
                self.close_pan()

    def start_ssh(self):
        self.dplug.plugins.network.ssh_status.Check(1)
        self.dplug.plugins.network.sshmenu.Enable(True)
        with self.create_ssh() as self.ssh:
            if self.ssh is False:
                print "Connection Failed"
            else:
                self.sftp = self.ssh.open_sftp()
                self.sftp.chdir(".")
    
    def stop_ssh(self):
        self.sftp.close()
        self.ssh.close()
        self.dplug.plugins.network.ssh_status.Check(0)
        self.dplug.plugins.network.sshmenu.Enable(False)

    @contextmanager
    def create_ssh(self):
        self.ssh = paramiko.SSHClient()
        self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.host = self.dplug.plugin_config.get("network","ssh_host")
        self.username = self.dplug.plugin_config.get("network","ssh_username")
        self.password = self.dplug.plugin_config.get("network","ssh_password")
        try:
           self.ssh.connect(self.host, username=self.username, password=self.password)
        except:
            yield False
        finally:
           yield self.ssh
    
    def get_upload_files(self):
        queue = []
        prev = self.local.GetFirstSelected()
        queue.append(self.local.GetItemText(prev))
        i = 1 #the first item is already read
        while i < self.local.GetSelectedItemCount():
            prev = self.local.GetNextSelected(prev)
            queue.append(self.local.GetItemText(prev))
            i+=1
        for x in queue:
            self.upload_file(x)
        self.virtual_update()
    
    def upload_file(self, file):
        if file.startswith("[F] "):
            OKDialog("SSH Upload", "Folder uploading currently not supported")
        else:
            self.sftp.put(os.path.join(self.filedir, file), os.path.join(self.sftp.getcwd(),file))
    
    def get_open_files(self):
        queue = []
        prev = self.local.GetFirstSelected()
        queue.append(self.local.GetItemText(prev))
        i = 1 #the first item is already read
        while i < self.local.GetSelectedItemCount():
            prev = self.local.GetNextSelected(prev)
            queue.append(self.local.GetItemText(prev))
            i+=1
        for x in queue:
            self.open_file(x)
    
    def open_file(self, x):
        self.gui.editfl(os.path.join(self.filedir, x))
    
    def get_download_files(self):
        queue = []
        prev = self.virtual.GetFirstSelected()
        queue.append(self.virtual.GetItemText(prev))
        i = 1 #the first item is already read
        while i < self.virtual.GetSelectedItemCount():
            prev = self.virtual.GetNextSelected(prev)
            queue.append(self.virtual.GetItemText(prev))
            i+=1
        for x in queue:
            self.download_file(x)
        self.local_update()
    
    def download_file(self, file):
        if file.startswith("[F] "):
            OKDialog("SSH Download", "Folder downloading currently not supported")
        else:
            self.sftp.get(os.path.join(self.sftp.getcwd(), file), os.path.join(self.filedir,file))
    
    def __ssh_factory(self, i):
        """Switcher help function"""
        def _switcher(event):
            """helper function"""
            return self.forward(i)
        return _switcher

    def Save(self):
        pass

    def install(self):
        """No ssh_network config yet"""
        if CSDialog("Init SSH","Your SSH configuration seems to be incomplete\nContinue to complete").GetValue():
            host = IODialog("SSH Host","SSH Host?").GetValue()
            if host:
                user = IODialog("SSH Username","SSH Username?").GetValue()
                if user:
                    pswd = IODialog("SSH Pass","SSH Password?", True).GetValue()
                    if pswd:
                        self.dplug.plugin_config.set("network", "ssh_host", host)
                        self.dplug.plugin_config.set("network", "ssh_username", user)
                        self.dplug.plugin_config.set("network", "ssh_password", pswd)
                        return True
        self.dplug.plugins.network.ssh_status.Check(0)
        return False
            
    """def ssh_shell(self):
        from terminal import TermEmulatorDemo
        shell = TermEmulatorDemo(self.gui.notebook, -1)
        shell.run_command("ls")
        self.gui.notebook.AddPage(shell, "Shell")"""

class Config(scrolled.ScrolledPanel):
    def __init__(self, parent, sth, i):
        self.dplug = i
        scrolled.ScrolledPanel.__init__(self, parent, -1)
        self.host = wx.TextCtrl(self)
        self.user = wx.TextCtrl(self)
        self.pswd = wx.TextCtrl(self, style = wx.TE_PASSWORD)
        try:
            self.host.SetValue(self.dplug.plugin_config.get("network","ssh_host"))
            self.user.SetValue(self.dplug.plugin_config.get("network","ssh_username"))
            self.pswd.SetValue(self.dplug.plugin_config.get("network","ssh_password"))
        except:
            pass
        sshsiz = wx.GridSizer(5, 2, 0, 0)
        sshsiz.Add(wx.StaticText(self, -1, "Host"))
        sshsiz.Add(self.host, -1, wx.ADJUST_MINSIZE)
        sshsiz.Add(wx.StaticText(self, -1, "Username"))
        sshsiz.Add(self.user)
        sshsiz.Add(wx.StaticText(self, -1, "Password"))
        sshsiz.Add(self.pswd)
        self.SetSizer(sshsiz)
        
        self.SetupScrolling()

    def Init(self):
        pass

    def Update(self):
        pass
